#ifndef BitWriter_H
#define BitWriter_H
//------------------------------------------------------------------------------
/* File: BitWriter.h
   
   Desc: This is the header file for the BitWriter. It outputs a ubyte8 to a file
         one all bits of the ubyte8 are full. It if constructed by specifying a
         a pointer to a file it needs to use for writing to. This meant to be
         created by the controller once per start and stop streaming. I don't
         open or close files. For the sake of speed no write error check is done
  
   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------

#include <GSTypes.h>
#include <stdio.h>

class CGSBitWriter
{
    private:
    protected:
        ubyte8  ucBuffer;
        ubyte8  ucBitCount;        
        FILE    *pFile;
        ulong32 lubyte8sWritten;

    public:

        CGSBitWriter( FILE *pOutputFile )
        {
            pFile         = pOutputFile;
            ucBuffer      = 0;
            ucBitCount    = 8;
            lubyte8sWritten = 0;
        }

        ~CGSBitWriter()
        {
        }

        void PutBits( ulong32 uiValue, ubyte8 ucCout );

        void AlignBits()
        {
            if( ucBitCount != 8 )PutBits( 0, ucBitCount );
        }

        ulong32 BitsWritten()
        {
            return ( 8*(lubyte8sWritten + 1) - ucBitCount  );
        }
};


inline void CGSBitWriter::PutBits( ulong32 uiValue, ubyte8 iCount )
{
    unsigned int mask = 1 << (iCount-1); // Select leftmost bit

    for( int i =0; i < iCount; i++ )
    {
        ucBuffer <<= 1;
        if( uiValue & mask  )
        {
            ucBuffer |= 1;
        }

        mask >>= 1; // Next Bit

        if( !( --ucBitCount ) ) // Buffer Full
        {
            putc( ucBuffer, pFile );
            ucBitCount = 8;
            lubyte8sWritten++;
        }
    }
}

#endif //#ifndef BitWriter_H
