#ifndef BLOCK8x8_H
#define BLOCK8x8_H
//------------------------------------------------------------------------------
/* File: Block8x8.h
  
   Desc: 8x8 Pixel Block Header File, Implementation is in MacroBlk to keep 
         functions inline
  
   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------

#include <GSTypes.h>


#define UnpackBytesToInt16 _asm movq      mm1,    [ebx]\
                           _asm movq      mm2,    mm1  \
                           _asm punpcklbw mm1,    mm0  \
                           _asm punpckhbw mm2,    mm0  \
                           _asm movq      [edx],  mm1  \
                           _asm movq      8[edx], mm2  \
                           _asm add       ebx,    ecx  \
                           _asm add       edx,    10h


    
class CGS8x8Block
{

friend class CGSMacroBlock;

protected:
        sint16    *sIntBlock;
        sint16    *pSourceBlock;
        sint16    imageWidth;
 static sint16A16 sQntMatrix[64];
 static sint16A16 sMltMatrix[64];
 static sint16A16 sAlternateMatrix[64];
 static sint16A16 sZigZagMatrix[64];
 static sint16A16 *sScanMatrix;
    
public:

        CGS8x8Block( sint16 iWidth, sint16 *pBlock, ubyte8 *pTopLeft )
        {
            imageWidth = iWidth;
            sIntBlock  = pBlock;
            Initialize( pTopLeft );
        }

        CGS8x8Block( sint16 *pBlock )
        {
            sIntBlock = pBlock;
        }


        void Initialize( ubyte8 *pTopLeftCorner )
        {   
            sint16  *pIntMatrix  = sIntBlock;
            __int32 iWidth = imageWidth;

            _asm
            {
                push    ebx                     ; pBuffer
                push    edx                     ; pIntMatrix
                push    ecx                     ; iWidth
                pxor    mm0,    mm0             ; Our Source For Zeros
                mov     ebx,    pTopLeftCorner  ;
                mov     edx,    pIntMatrix      ;
                mov     ecx,    iWidth          ;

                UnpackBytesToInt16              ; (0)
                UnpackBytesToInt16              ; (1)
                UnpackBytesToInt16              ; (2)
                UnpackBytesToInt16              ; (3)
                UnpackBytesToInt16              ; (4)
                UnpackBytesToInt16              ; (5)
                UnpackBytesToInt16              ; (6)
                UnpackBytesToInt16              ; (7)

                pop     ecx                     ;
                pop     edx                     ;
                pop     ebx                     ;
            }
        }

        void fDCT();
        void iDCT();
        void Write();
        void IntraQuantizeAndPutBits( uint16 cc );
        void IntraQuantize();
        void Quantize();
        void QuantizeP4();
 static void FindMultiplier();
 static void IntraDequantize( sint16 *pSrc, sint16 *pDest );
 static void IntraDequantizeP4( sint16 *pSrc, sint16 *pDest );
 static void SetAlternateScan(){ sScanMatrix = sAlternateMatrix; }
 static void SetZigZagScan(){ sScanMatrix = sZigZagMatrix; }
};




#endif //#ifndef BLOCK8x8_H