#ifndef MacroBlk_H
#define MacroBlk_H
//------------------------------------------------------------------------------
/* File: MacroBlk.h
  
   Desc: This file imlements a three pland bitmap MacroBlock
  
   Warning: Some counters are unsinged char, restircting MacroBlock size to 2040x2040
            which we did on purpose, considering even ATSC is only 1280x720 
  
   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------

#include <GSTypes.h>
#include <Block8x8.h>


class CGSMacroBlock
{
    friend class CGSPicture;
    friend class CGSIPicture;
    friend class CGSPPicture;
    friend class CGSBPicture;
    friend class CGSMotionCalculator;

    protected:
        bool           isFramDCT;
        bool           isCodedBlockPattern;
        bool           isSkipped;
        bool           isDebug;
        MacroBlockType Type;
        MotionVecType  VectorType;
        int            MV[2][2][2];
        int            MVFieldSelect[2][2];
        int            DMV[2];
        ubyte8         *pBlockTL[12]; // These are pointers to the Luminance Plane
        sint16         *pBlocks[12];  // These are blocks, memory is allocated by GSImage
        double         ActivityMeasure;

    public:

// The following are valid in context of the Sequence.
        static int    imageWidth;
        static int    cc[12];
        static int    dc_dct_pred[3];
        static CGSVLCoder *pVLCoder;
        static int     NumberOfBlocks;
        static int     DCDivider;
        static int     QuantizationShifter;

        CGSMacroBlock()
        {
            isFramDCT           = true;
            isCodedBlockPattern = false;
            isSkipped           = false;
            isDebug             = false;
            Type                = IntraBlock;
            VectorType          = Frame16x8Vector;
            ActivityMeasure     = 0;
            MV[0][1][0]         =
            MV[0][1][1]         =
            MV[0][0][0]         =
            MV[0][0][1]         =
            MV[1][1][0]         =
            MV[1][1][1]         =
            MV[1][0][0]         =
            MV[1][0][1]         = 0;
            MVFieldSelect[0][0] = 0;
            MVFieldSelect[0][1] = 0;
            MVFieldSelect[1][0] = 0;
            MVFieldSelect[1][1] = 0;
            DMV[2] = DMV[1]     = 0;
        }

        ~CGSMacroBlock()
        {
        }

        void SetDebug( bool bDebug ){ isDebug = bDebug; }
        
        void Finalize()
        {
        }

        void SetData()
        {
            int i;
            for( i = 0; i < 4; i++ )
            {
                CGS8x8Block Block( imageWidth, pBlocks[i], pBlockTL[i] );
            }
            for( ; i < NumberOfBlocks; i++ )
            {
                CGS8x8Block Block( imageWidth/2, pBlocks[i], pBlockTL[i] );
            }
        }


        void Variance()
        {
        }

        void IntraQuantizeAndPutBits();
        void Write();

        // Static, setup functions, valid in context of a CGSSequence

        static void SetVLCoder( CGSVLCoder &rAVLCoder )
        { 
            pVLCoder = &rAVLCoder;
        }

        static void SetDCPrecision( ubyte8 ucPrecision )
        {
            DCDivider = 8 >> ucPrecision;
        }

        static void SetChromaFormat( ubyte8 ucChromaFormat )
        {
            NumberOfBlocks = 4 + (2 * ucChromaFormat);
        }

        static void SetQuantizationShifter( ubyte8 ucMultiplier )
        {
            switch( ucMultiplier )
            {
                case 2:
                default:
                    QuantizationShifter = 1;
                    break;
                case 4:
                    QuantizationShifter = 2;
                    break;
                case 8:
                    QuantizationShifter = 3;
                    break;
                case 16:
                    QuantizationShifter = 4;
                    break;
            }
        }
};

#endif //#ifndef GSMacroBlock_H
