#ifndef MotnCalc_H
#define MotnCalc_H
//------------------------------------------------------------------------------
/* File: MotnCalc.h
  
   Desc: Header File for Motion Calculator, based on Intel's AP-940 Sample Code
  
  
   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------

#include <GSTypes.h>
#include <MacroBlk.h>


#define CalcSADofRow  _asm add     edx, ecx    \
                      _asm add     ebx, ecx    \
                      _asm movq    mm1, [edx]  \
                      _asm movq    mm2, 8[edx] \
                      _asm psadbw  mm1, [ebx]  \
                      _asm psadbw  mm2, 8[ebx] \
                      _asm paddw   mm6, mm1    \
                      _asm paddw   mm7, mm2    


class CGSMotionCalculator
{
    protected:
        sint16 PMV[2][2][2];
        uint16 iWidthInPixels;
        uint16 iHeightInPixels;
static  ubyte8 FCode;
static  ubyte8 SearchRange;
static  sint16 ErrorThreshold;
    public:

        CGSMotionCalculator( uint16 iWidth, uint16 iHeight )
        {
            iWidthInPixels  = iWidth;
            iHeightInPixels = iHeight;
            ClearPredictors();
        }

        void ClearPredictors()
        {
            PMV[0][1][0] =
            PMV[0][1][1] =
            PMV[0][0][0] =
            PMV[0][0][1] =
            PMV[1][1][0] =
            PMV[1][1][1] =
            PMV[1][0][0] =
            PMV[1][0][1] = 0;
        }

static  void SetFCode( ubyte8 ucFCode )
        {
            if( ucFCode > 3 )ucFCode = 3; // Limit Range to 16
            FCode = ucFCode;
            SearchRange = 1 << (FCode+1);
        }

static  inline sint16 GetErrorThreshold(){ return ErrorThreshold; };
static  inline void   SetErrorThreshold( sint16 aThreshold ){ ErrorThreshold = aThreshold; }


        inline int MotionError( ubyte8* frame, ubyte8* block, __int32 iWidth )
        {
            int iVal;

            _asm
            {
                   pxor   mm6, mm6                           ;mm6 mm7 two buffers for final sum
                   pxor   mm7, mm7                           ; of abs. differences
                   mov    edx, frame                         ;pointer to  reference frame into edx.
                   mov    ebx, block                         ;pointer to  estimated block into ebx
                   mov    ecx, iWidth                        ;width of the frames, ref, and the macroblock
                   
                   ;reading  unaligned data with 2 simple MOVQ's every line  
                   ;aligned data strictly from memory
                   ;PSADBW - suming the absolute diferences of 8 pairs of pixels
                   ;edx - pointer to reference block line beginning(unaligned)
                   ;ebx - pointer to estimated block line beginning (aligned)
                   ;iWidth - the distance in bytes from pixel to pixel just below it
  
                   movq    mm1,[edx]                         ; First Row
                   movq    mm2,8[edx]
                   psadbw  mm1, [ebx]                   
                   psadbw  mm2,8[ebx]                 
                   paddw   mm6, mm1
                   paddw   mm7, mm2
                    
                   CalcSADofRow                              ; (1 )
                   CalcSADofRow                              ; (2 )
                   CalcSADofRow                              ; (3 )
                   CalcSADofRow                              ; (4 )
                   CalcSADofRow                              ; (5 )
                   CalcSADofRow                              ; (6 )
                   CalcSADofRow                              ; (7 )
                   CalcSADofRow                              ; (8 )
                   CalcSADofRow                              ; (9 )
                   CalcSADofRow                              ; (10)
                   CalcSADofRow                              ; (11)
                   CalcSADofRow                              ; (12)
                   CalcSADofRow                              ; (13)
                   CalcSADofRow                              ; (14)
                   CalcSADofRow                              ; (15)

                   paddd mm7,mm6
                   movd  iVal, mm7
            }
            return iVal;
        }


        inline int MotionErrorP4( ubyte8* frame, ubyte8* block, __int32 iWidth )
        {
            int iVal;

            _asm
            {
                   pxor   xmm6, xmm6                         ;mm6 mm7 two buffers for final sum
                   pxor   xmm7, xmm7                         ; of abs. differences
                   mov    edx, frame                         ;pointer to  reference frame into edx.
                   mov    ebx, block                         ;pointer to  estimated block into ebx
                   mov    ecx, iWidth                        ;width of the frames, ref, and the macroblock
                   
                   ;reading  unaligned data with 2 simple MOVQ's every line  
                   ;aligned data strictly from memory
                   ;PSADBW - suming the absolute diferences of 8 pairs of pixels
                   ;edx - pointer to reference block line beginning(unaligned)
                   ;ebx - pointer to estimated block line beginning (aligned)
                   ;iWidth - the distance in bytes from pixel to pixel just below it
  
                   movdqu  xmm1, [edx]                      ; First Row
                   psadbw  xmm1, [ebx]                   
                   paddw   xmm6, xmm1
                    

                   movd  iVal, mm7
            }
            return iVal;
        }



        inline void CalcMotionByFullSearch( CGSMacroBlock *pMB, ubyte8 *pRefFrame, int iX, int jY )
        {
            int  iBestDiff = 0x7fffffff, iCalcDiff = 0x7fffffff, i, j;
            bool bFoundMatch = false;

            int    *pX     = &pMB->MV[0][0][0];
            int    *pY     = &pMB->MV[0][0][1];
            ubyte8 *pMBTop = pMB->pBlockTL[0];
            int    iStart  = iX - SearchRange;
            int    jStart  = jY - SearchRange;
            int    iEnd    = iX + SearchRange + 16;
            int    jEnd    = jY + SearchRange + 16;
            int    iLimit  = iWidthInPixels   - 16;
            int    jLimit  = iHeightInPixels  - 16;
            
            if( iStart < 0      )iStart = 0;
            if( jStart < 0      )jStart = 0;
            if( iEnd   > iLimit )iEnd   = iLimit;
            if( jEnd   > jLimit )jEnd   = jLimit;

        
            // Now do the rest

            for( i = iStart; i < iEnd; i++ )
            {
                for( j = jStart; j < jEnd; j++ )
                {   
                    iCalcDiff = MotionError( (pRefFrame+(j*iWidthInPixels)+i), pMBTop, iWidthInPixels );
                    if( iCalcDiff < iBestDiff )
                    {
                        iBestDiff = iCalcDiff;
                        if( iBestDiff <= ErrorThreshold )
                        {
                            bFoundMatch = true;
                            goto ExitMotionCalc;
                        }
                    }
                }
            }

            ExitMotionCalc:
            {
                if( bFoundMatch )
                {
                    *pX = i - iX;
                    *pY = j - jY;

                    if( *pX || *pY )
                    {
                        pMB->Type = FwdBlock;
                    }else
                    {
                        pMB->Type = SkipBlock;
                    }
                }else
                {
                    pMB->Type = IntraBlock;
                }
            }
        }
};

#endif //#ifndef MotnCalc_H
