#ifndef Picture_H
#define Picture_H
//------------------------------------------------------------------------------
/* File: GSPicture.h
  
   Desc: This file imlements a three pland bitmap image
  
   Warning: Some counters are unsinged char, restircting image size to 2040x2040
            which we did on purpose, considering even ATSC is only 1280x720 
  
   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------

#include <GSTypes.h>


class CMPEGPictureHeader
{
public:
    ulong32 PictureStartCode;
    ubyte8 ExtraBit;

    CMPEGPictureHeader()
    {
        PictureStartCode = 0x00000100;
        ExtraBit         = 1;
    }

    void Write( CGSBitWriter &, PictureType eType, uint16 iTemporalReference, uint16 iVBDelay );
};


class CMPEGExtPictureHeader
{
public:
    ulong32 ExtStartCode;          //32
    ubyte8 ExtensionID;           //4
    ubyte8 FwdHorizonatlFCode;    //4
    ubyte8 FwdVerticalFCode;      //4
    ubyte8 BckdHorizonatlFCode;   //4
    ubyte8 BckVerticalFCode;      //4
    ubyte8 DCCoeffPrecision;      //2
    ubyte8 PictureStructure;      //2
    ubyte8 TopFirst;              //1
    ubyte8 FramePredictDCT;       //1
    ubyte8 ConcealmentVectors;    //1
    ubyte8 QScaleType;            //1
    ubyte8 IsIntraVLCFormat;      //1
    ubyte8 IsAlternateScan;       //1
    ubyte8 ShouldRepeatFirst;     //1
    ubyte8 IsChroma420;           //1
    ubyte8 IsProgressiveFrame;    //1
    ubyte8 CompsiteDisplayFlag;   //1
    uint16 DCPredictorResetVal;   // Not Transmitted

    CMPEGExtPictureHeader()
    {
        ExtStartCode        = 0x000001B5; //All Sequence Extensions
        ExtensionID         = 8;          // ID of this     
        FwdHorizonatlFCode  = 0xFF;       // MPEG2 - All bits set for I 
        FwdVerticalFCode    = 0xFF;       // MPEG2 - All bits set for I
        BckdHorizonatlFCode = 0xFF;       // MPEG2 - All bits set for I
        BckVerticalFCode    = 0xFF;       // MPEG2 - All bits set for I
        DCCoeffPrecision    = 0;          // Eight Bit Precision, Divide DC by 8
        PictureStructure    = 3;          // Can't deal with interlaced yet
        TopFirst            = 0;          // Can't deal with interlaced yet 
        FramePredictDCT     = 1;          // Can't deal with interlaced yet  
        ConcealmentVectors  = 0;          // Need to change when use MVs
        QScaleType          = 1;          // Use Non-linear Quantizer Table 7-6
        IsIntraVLCFormat    = 1;          // Table B-15, statistically Optimized, for Intra Coded Blocks
        IsAlternateScan     = 1;          // Since we will eventually do interlaced video, use alternate scan
        ShouldRepeatFirst   = 0;          // Force just one frame for now, as we don't do interlaced
        IsChroma420         = 0;          // We are using 4:2:2 by default
        IsProgressiveFrame  = 1;          // Force Non-interlaced for now
        CompsiteDisplayFlag = 0;          // This is a meaningless thing!!!
        DCPredictorResetVal = 128;        // Assuming 8 bit precision;
    }

    void SetZigZagScan();
    void SetAlternateScan();
    void ClearIntraVLCFormat();
    void SetIntraVLCFormat();
    void Write( CGSBitWriter & );
    void SetChroma420( ubyte8 is420 ){ IsChroma420 = is420; };
    void SetDCPrecision( ubyte8 ucNumBits );
    void SetFCodes( PictureType ePicture, ubyte8 ucCode );
};


class CMPEGSlice
{
public:
    ulong32 SliceStartCode;

    CMPEGSlice() { SliceStartCode = 0x00000101; }

    void Write( CGSBitWriter &BitWriter, ubyte8 ucRow, ubyte8 ucScaleCode );
};



class CGSPicture
{
    friend class CGSSequence;
    friend class CGSIPicture;
    friend class CGSPPicture;
    friend class CGSBPicture;

    private:
        void InitializeMacroBlocks();
        void CreateMacroBlockBlocks();


    protected:
        bool    isInitialized;
        ubyte8  imageWidthInMacroBlocks;
        ubyte8  imageHeightInMacroBlocks;
        ubyte8  imageWidthInBlocks;
        ubyte8  imageHeightInBlocks;
        uint16  imageWidthInPixels;
        uint16  imageHeightInPixels;
        uint16  numberOfMacroBlocks;
        uint16  numberOfBlocks;
        ulong32 numberOfPixels;
        ubyte8 *pPlanarImageRoot;
        ubyte8 *pPlanarImage;
        ubyte8 *pYPlane;
        ubyte8 *pCbPlane;
        ubyte8 *pCrPlane;
        sint16 *pBlockMemoryRoot;
        sint16 *pBlockMemory;
        CGSMacroBlock *pMacroBlocks;

        PictureType           eType;
        CMPEGPictureHeader    PictureHeader;
        CMPEGExtPictureHeader ExtPictureHeader;
        CMPEGSlice            Slice;

//      The following static objects valid in context of a Sequence.

static  CGSMotionCalculator   *pMotionCalculator;
static  CGSVLCoder            *pVLCoder;
static  CGSBitWriter          *pBitWriter;

    public:

        CGSPicture()
        {
            pPlanarImage  = pPlanarImageRoot = 0;
            pMacroBlocks  = 0;
            pBlockMemory  = pBlockMemoryRoot = 0;
            eType         = None;
            isInitialized = false;
        }


        void Initialize( uint16 biWidth, uint16 biHeight );
        void Finalize();

        ~CGSPicture()
        {
            if( isInitialized )Finalize();
        }

        

virtual void ConstructImageFromMacroBlocks() = 0;
virtual void Debug();
virtual void Predict( CGSPicture *pBeforeMe, CGSPicture *pAfterMe ) = 0;
virtual void OutputImage( ubyte8 *pRGBImage ); // Override this to do anything but a YUY2 output
virtual void SetData() = 0;
virtual void SplitPlanes( ubyte8 *pRGBImage ); // Epectation is that 1+0.5+0.5 planes for 4:2:2 is populated
virtual void Write( ulong32 uPictureCount ) = 0;

static  void SetBitWriter( CGSBitWriter &rBitWriter ){ pBitWriter = &rBitWriter; }
static  void SetVLCoder  ( CGSVLCoder   &rVLCoder   ){ pVLCoder   = &rVLCoder;   }
static  void SetMotionCalculator( CGSMotionCalculator &rMotionCalculator ) { pMotionCalculator = &rMotionCalculator; }

};

#endif //#ifndef Picture_H
