//------------------------------------------------------------------------------
/* File: GSIPicture.cpp
  
   Desc: Implementation File for Three Plane Bitmap Image
  
   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------

#include <IPicture.h>
#include <VLCoder.h>

// The code fragment below, moves 4 shorts to mm1, and uses packuswb to put another four shorts
// packed to become a total of 8 bytes of the image from 8 integers of a row of a block of an MB
// We need to determine if on a P4, it is any better to use xmm registers. It will generate 16 bytes
// but we will have to reorganize construction of the macroblock. That will make set up for fDCT and
// iDCT setup slower, as we will have to reconstruct a block.


#define ConvertInt16ToBytes _asm movq     mm1,   [ebx] \
                            _asm packuswb mm1,   8[ebx]\
                            _asm movq     [edx], mm1   \
                            _asm add      ebx,   10h   \
                            _asm add      edx,   ecx

static sint16A16 anAlignedBlock[64];


void CGSIPicture::ConstructImageFromMacroBlocks()
{
    ubyte8 i, j, n;

    for( i = 0; i < imageHeightInMacroBlocks; i++ ) // For Each Macroblock Row
    {
        CGSMacroBlock *pMB = pMacroBlocks + i * imageWidthInMacroBlocks;

        for( j = 0; j < imageWidthInMacroBlocks; j++ ) // For Each Macroblock Column
        {
            ubyte8 *pY  = pYPlane;
                    pY += 16*i*imageWidthInPixels; // Every row of macroblock == 16 row of pels 
                    pY += 16*j;                    // Each colum of mactoblock is 16 wide
            for( n = 0; n < 4; n++ )               // For Each Luminance Block
            {
                sint16 *pTopLeftOfBlock = anAlignedBlock;
#ifdef GSP4
                CGS8x8Block::IntraDequantizeP4( pMB->pBlocks[n], pTopLeftOfBlock );
#else
                CGS8x8Block::IntraDequantize( pMB->pBlocks[n], pTopLeftOfBlock );
#endif          
                pY += ((n%2)*8); // Move right eight before writing odd blocks

                ubyte8 *pDest = pY;
                sint16 *pSrc  = pTopLeftOfBlock;
                int iWidth    = imageWidthInPixels;

                _asm
                {
                    ; Save Registers
                    push     ebx            ; Pointer to pSrc
                    push     edx            ; Pointer to pDest
                    push     ecx            ; iWidth

                                            ; Get Setup

                    mov      edx, pDest     ;
                    mov      ebx, pSrc      ;
                    mov      ecx, iWidth    ;
                    
                                            ; Do the move

                    ConvertInt16ToBytes     ; (0)
                    ConvertInt16ToBytes     ; (1)
                    ConvertInt16ToBytes     ; (2)
                    ConvertInt16ToBytes     ; (3)
                    ConvertInt16ToBytes     ; (4)
                    ConvertInt16ToBytes     ; (5)
                    ConvertInt16ToBytes     ; (6)
                    ConvertInt16ToBytes     ; (7)
                                                
                    ; Restore the registers we used

                    pop  ecx            ;
                    pop  edx            ;
                    pop  ebx            ;
                }

                pY -= ((n%2)*8); // Move back to the left corner after odd blocks
                pY += ((n%2)*8*imageWidthInPixels); // When done with Block 1&3,move down 8 Rows
            }

            pMB++; // Do the next Macroblock

        }
    }
}



void CGSIPicture::Write( ulong32 uPictureCount )
{
    uint16 uTemporalReference = (uint16)(uPictureCount%1024);
    ubyte8  uMBRow, uMBCol;
    ubyte8  ucQuantizerScaleCode = 8;// We are not doing any rate control.
    CGSMacroBlock *pMacroBlock = pMacroBlocks;

    PictureHeader.Write( *pBitWriter, I, uTemporalReference, 0xFFFF ); // VBV Delay is questionable!
    
    ExtPictureHeader.SetDCPrecision( 10 );
    ExtPictureHeader.SetFCodes( I, 7 );
    ExtPictureHeader.Write( *pBitWriter );

    for( uMBRow = 0; uMBRow < imageHeightInMacroBlocks; uMBRow++ )
    {
        Slice.Write( *pBitWriter, uMBRow, ucQuantizerScaleCode );
        CGSMacroBlock::dc_dct_pred[0] = CGSMacroBlock::dc_dct_pred[1]
                                      = CGSMacroBlock::dc_dct_pred[2]
                                      = ExtPictureHeader.DCPredictorResetVal;

        for( uMBCol = 0; uMBCol < imageWidthInMacroBlocks; uMBCol++ )
        {
            pVLCoder->PutMacroBlockAddrInc( 1 ); // For I picture, we need to write all of them
            pVLCoder->PutMacroBlockType( I, pMacroBlock->Type );
            pMacroBlock->IntraQuantizeAndPutBits();
            pMacroBlock++;
        }
    }
}



/*          

void CGSIPicture::ConstructChrominanceImageFromMacroBlocks()
{
    ubyte8 i, j, k, n;
    sint16 aBlock[64];

    for( i = 0; i < imageHeightInMacroBlocks; i++ ) // For Each Macroblock Row
    {
        CGSMacroBlock *pMB = pMacroBlocks + i * imageWidthInMacroBlocks;

        for( j = 0; j < imageWidthInMacroBlocks; j++ ) // For Each Macroblock Column
        {
            ubyte8 *pCb  = pCbPlane;               // Top Left Corner
                    pCb += 8*i*imageWidthInPixels; // For each row advance half image width * 16, eqv 8
                    pCb += 8*j;                    // For each macroblock row, slide 8 over to right

            for( n = 4; n < pMB->NumberOfBlocks; n += 2 )               // For Each Cb Block
            {
                sint16 *pTopLeftOfBlock = aBlock;
                CGS8x8Block::IntraDequantize( pMB->pBlocks[n], pTopLeftOfBlock );
                
                for( k = 0; k < 8; k++ ) // For each row in the block
                {
                    ubyte8 *pDest = pCb + k*imageWidthInPixels/2; // For each row, step down imageWidthInPixels
                    for( sint16 *pSrc = pTopLeftOfBlock; pSrc < pTopLeftOfBlock+8; ) // For each pixel across
                    {
                        *pDest++ = (ubyte8)(*pSrc++);
                    }
                    pTopLeftOfBlock += 8; // Go to next row in the block.
                }
                pCb += (4*imageWidthInPixels); // Move down 8 * imageWidthInPixels, halfwidth, so 4
            }



            ubyte8 *pCr  = pCrPlane;
                    pCr += 8*i*imageWidthInPixels;
                    pCr += 8*j;

            for( n = 5; n < pMB->NumberOfBlocks; n += 2 )               // For Each Cr Block
            {
                sint16 *pTopLeftOfBlock = aBlock;
                CGS8x8Block::IntraDequantize( pMB->pBlocks[n], pTopLeftOfBlock );

                for( k = 0; k < 8; k++ ) // For each row in the block
                {
                    ubyte8 *pDest = pCr + k*imageWidthInPixels/2; // For each row, step down imageWidthInPixels
                    for( sint16 *pSrc = pTopLeftOfBlock; pSrc < pTopLeftOfBlock+8; ) // For each pixel across
                    {
                        *pDest++ = (ubyte8)(*pSrc++);
                    }
                    pTopLeftOfBlock += 8; // Go to next row in the block.
                }
                pCr += (4*imageWidthInPixels);
            }

        }
    }
}
*/                      
