//------------------------------------------------------------------------------
/* File: MacroBlk.cpp
  
   Desc: Implementation File for MacroBlock & Block
  
   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------

#include <MacroBlk.h>
#include <VLCoder.h>
int CGSMacroBlock::QuantizationShifter = 1;
int CGSMacroBlock::DCDivider           = 8;
int CGSMacroBlock::NumberOfBlocks      = 8; // Assume we user 4:2:2 by default.
int CGSMacroBlock::imageWidth;
int CGSMacroBlock::cc[12] = {0,0,0,0,1,2,1,2,1,2,1,2};
int CGSMacroBlock::dc_dct_pred[3];
CGSVLCoder *CGSMacroBlock::pVLCoder = 0;



sint16A16 *CGS8x8Block::sScanMatrix = sAlternateMatrix;

sint16A16 CGS8x8Block::sQntMatrix[64]=
{
     8, 16, 19, 22, 26, 27, 29, 34, 
    16, 16, 22, 24, 27, 29, 34, 37, 
    19, 22, 26, 27, 29, 34, 34, 38, 
    22, 22, 26, 27, 29, 34, 37, 40, 
    22, 26, 27, 29, 32, 35, 40, 48, 
    26, 27, 29, 32, 35, 40, 48, 58, 
    26, 27, 29, 34, 38, 46, 56, 69, 
    27, 29, 35, 38, 46, 56, 69, 83
};

sint16A16 CGS8x8Block::sAlternateMatrix[64]=
{
     0,  8, 16, 24,  1,  9,  2, 10,
    17, 25, 32, 40, 48, 56, 57, 49,
    41, 33, 26, 18,  3, 11,  4, 12,
    19, 27, 34, 42, 50, 58, 35, 43,
    51, 59, 20, 28,  5, 13,  6, 14,
    21, 29, 36, 44, 52, 60, 37, 45,
    53, 61, 22, 30,  7, 15, 23, 31,
    38, 46, 54, 62, 39, 47, 55, 63
};

sint16A16 CGS8x8Block::sZigZagMatrix[64]=
{
     0,  1,  8, 16,  9,  2,  3, 10,
    17, 24, 32, 25, 18, 11,  4,  5,
    12, 19, 26, 33, 40, 48, 41, 34,
    27, 20, 13,  6,  7, 14, 21, 28,
    35, 42, 49, 56, 57, 50, 43, 36,
    29, 22, 15, 23, 30, 37, 44, 51,
    58, 59, 52, 45, 38, 31, 39, 46,
    53, 60, 61, 54, 47, 55, 62, 63
};


sint16A16 CGS8x8Block::sMltMatrix[64];


#include <ippi.h>

inline void CGS8x8Block::fDCT()
{
    ippiDCT8x8Fwd_16s_C1I( sIntBlock );
}

inline void CGS8x8Block::iDCT()
{    
    ippiDCT8x8Inv_16s_C1I( sIntBlock );
}


/*

Taken from Peter L. Mongomery et. al., the below algorithm computes a multiplier such that
if m, d, L are integers such that d > 0, and 0 <= m*d - 2**(N+L-1) <= 2**L and n is an
integer such that -2**(N-1) <= n <= 2**(N-1)-1, we define qo = m*n/( 2**(N+L-1) ) then,

TRUNC(n/d) = qo   if n >= 0 
TRUNC(n/d) = q0+1 if n <  0;

Since in the quantization process, we use DCT coefficients that are saturated to

-2048 <= F(u,v) <= 2047, I chose N = 12, and L = 7 (for division by 83). Since our
least divider is 16, as W(0,0) is not used, m can be shown to fit in the range 
-32768 <= m <= 32767, thus m can be stored in an 16bit signed integer.

Quantizer scales have higher multiplier of 16, which means we will need full range of
short int for multiplication with non-linear quantization scale. This will induce some
error in division, the quotient will be off with the following statistics... for the default
MPEG2 quantization matrix.

For Range -2048 <= n <= 2047, exact, no errors
For Range -4096 <= n <= 4096, exact, no errors

For Range -8192 <= n <= 8191
There were 478 Errors (Max =  1) in 262112 Computations
Percentage Error is  0.0018 RMS = 0.00008

For Range -16384 <= n <= 16383
There were 11656 Errors (Max =  1) in 524256 Computations
Percentage Error is  0.0222 RMS = 0.00021

For Range -32768 <= n <= 32767
There were 97914 Errors (Max =  1) in 1048544 Computations
Percentage Error is  0.0934 RMS = 0.00030

At the cost of an extra addition, one could increase the precision to
There were 764 Errors (Max =  1) in Computations in 524256 Computations
Percentage Error is  0.0015 RMS = 0.00005 in the range, -16384<=n<=16383,
increasing the Quantizer scale range to 16384/2048 = 8. I have chosen not
to pay the extra cost of the addition.


*/


void CGS8x8Block::FindMultiplier()
{
    sint16 *pQuantizer = sQntMatrix, *pMultiplier = sMltMatrix;
    sint16 *pEndOfQuantizer = pQuantizer+64;

    while( pQuantizer < pEndOfQuantizer )
    {
        int m, L, iExp, N = 12, l, iAddExp, iAdd, Divider = *pQuantizer++;


        // Find l = log2( Divider );

        for( int i = 1; i < 15; i++ )
        { 
            iAdd = 1 << i; // ( 2**i )
            L = i;
            if( Divider <= iAdd )break;
        }
    
        l = L; // Log of the number we are dealing with

        L = 7; // Our max divider is 83, so set this to 7

        // Let us add, 2**( L-l) /d to the multiplier

        iAddExp = L - l;
        iAdd    = 1 << iAddExp;
        iAdd   /= Divider;

        iExp = N + L - 1;
        m    = 1 << iExp;
        m   /= Divider;
        m++;

        //    printf( "Quantizer = %4.4d\tMultiplier = %6.6d\tShift By %1.1d\n", Divider, m,  N+L-1 );

        *pMultiplier++ = (sint16)m;
    }
}



void CGS8x8Block::QuantizeP4()
{
    __int64 qScaleShifter = CGSMacroBlock::QuantizationShifter;
    sint16  *pM     = sMltMatrix;
    sint16  *pBlock = sIntBlock;
    _asm
    {
        push    edx                     ; To store the pointer to the block
        push    ecx                     ; qScaleShifter
        push    ebx                     ; To store the pointer to the multiplier
                                        ; Get setup
        movq    mm0,    qScaleShifter   ;
        mov     ebx,    pM              ; Multiplier Matrix
        mov     ecx,    pBlock          ; Pointer to the block
        mov     edx,    ecx             ; Pointer to the block
        add     ecx,    80h             ; + 64, integers in an 8x8 block to end of block, 128 bytes
                                        ;
    DoDivide:                           ; The first eight integers
                                        ;
                                        ;
        movq2dq xmm1,    mm0            ; Fill All Odd Registers with qScaleShifter
        movdqa  xmm3,    xmm1           ; Fill All Odd Registers with qScaleShifter
        movdqa  xmm5,    xmm3           ; Fill All Odd Registers with qScaleShifter
        movdqa  xmm7,    xmm5           ; Fill All Odd Registers with qScaleShifter

        movdqa  xmm0,    [edx]          ; Get the first eight integers
        psllw   xmm0,    xmm1           ;
        movdqa  xmm1,    xmm0           ; For sign test and adding
        pmulhw  xmm0,    [ebx]          ; Do the multiply, get the high word MULSH( m, n )
        psrlw   xmm1,    15             ; Extract the sign bit of the numerator
        psraw   xmm0,    2              ; Right shift the quotient by two, 16+2 = 18 = N+L-1
        paddw   xmm0,    xmm1           ; Add the sign bit
        movdqa  [edx],   xmm0           ; Done with 0 to 7
                                        ;
        movdqa  xmm2,    [edx+16]       ;
        psllw   xmm2,    xmm3           ;
        movdqa  xmm3,    xmm2           ; For sign test and adding
        pmulhw  xmm2,    [ebx+16]       ; Do the multiply, get the high word MULSH( m, n )
        psrlw   xmm3,    15             ; Extract the sign bit of the numerator
        psraw   xmm2,    2              ; Right shift the quotient by two, 16+2 = 18 = N+L-1
        paddw   xmm2,    xmm3           ; Add the sign bit
        movdqa [edx+16], xmm2           ; Done with 8 to 15
                                        ;
        movdqa  xmm4,    [edx+32]       ;
        psllw   xmm4,    xmm5           ;
        movdqa  xmm5,    xmm4           ; For sign test and adding
        pmulhw  xmm4,    [ebx+32]       ; Do the multiply, get the high word MULSH( m, n )
        psrlw   xmm5,    15             ; Extract the sign bit of the numerator
        psraw   xmm4,    2              ; Right shift the quotient by two, 16+2 = 18 = N+L-1
        paddw   xmm4,    xmm5           ; Add the sign bit
        movdqa [edx+32], xmm4           ; Done with 16 to 23
                                        ;
        movdqa  xmm6,    [edx+48]       ;
        psllw   xmm6,    xmm7           ;
        movdqa  xmm7,    xmm6           ; For sign test and adding
        pmulhw  xmm6,    [ebx+48]       ; Do the multiply, get the high word MULSH( m, n )
        psrlw   xmm7,    15             ; Extract the sign bit of the numerator
        psraw   xmm6,    2              ; Right shift the quotient by two, 16+2 = 18 = N+L-1
        paddw   xmm6,    xmm7           ; Add the sign bit
        movdqa [edx+48], xmm6           ; Done with 24 to 32

        add     ebx,    40h             ; 64 bytes, 32 integers
        add     edx,    40h             ;
        cmp     edx,    ecx             ; Did we reach end of block
        jl      DoDivide                ; Go back and to 16 to 63
                                        ;
                                        ; Clean Up
        pop     ebx                     ;
        pop     ecx                     ;
        pop     edx                     ;
    }       
}




void CGS8x8Block::Quantize()
{
    register __int64 qScaleShifter = CGSMacroBlock::QuantizationShifter;
    sint16  *pM     = sMltMatrix;
    sint16  *pBlock = sIntBlock;
    _asm
    {
        push    edx                     ; To store the pointer to the block
        push    ecx                     ; end of the block
        push    ebx                     ; To store the pointer to the multiplier
                                        ; Get setup            
        mov     ebx,    pM              ; Multiplier Matrix
        mov     ecx,    pBlock          ; Pointer to the block
        mov     edx,    ecx             ; Pointer to the block
        add     ecx,    80h             ; + 64, integers in an 8x8 block to end of block, 128 bytes
                                        ;
    DoDivide:                           ; The first eight integers
                                        ;
        movq    mm0,    [edx]           ;
        movq    mm2,    [edx+8]         ;
        psllw   mm0,    qScaleShifter   ;
        psllw   mm2,    qScaleShifter   ;
        movq    mm1,    mm0             ; For sign test and adding
        movq    mm3,    mm2             ; For sign test and adding
        pmulhw  mm0,    [ebx]           ; Do the multiply, get the high word MULSH( m, n )
        pmulhw  mm2,    [ebx+8]         ; Do the multiply, get the high word MULSH( m, n )
        psrlw   mm1,    15              ; Extract the sign bit of the numerator
        psrlw   mm3,    15              ; Extract the sign bit of the numerator
        psraw   mm0,    2               ; Right shift the quotient by two, 16+2 = 18 = N+L-1
        psraw   mm2,    2               ; Right shift the quotient by two, 16+2 = 18 = N+L-1
        paddw   mm0,    mm1             ; Add the sign bit
        paddw   mm2,    mm3             ; Add the sign bit
        movq    [edx],  mm0             ; Done with 0 to 3
        movq   [edx+8], mm2             ; Done 4 to 7
                                        ;
                                        ;
        movq    mm4,    [edx+16]        ; Next Eight Integers  
        movq    mm6,    [edx+24]        ;
        psllw   mm4,    qScaleShifter   ;
        psllw   mm6,    qScaleShifter   ;
        movq    mm5,    mm4             ; For sign test and adding
        movq    mm7,    mm6             ; For sign test and adding
        pmulhw  mm4,    [ebx+16]        ; Do the multiply, get the high word MULSH( m, n )
        pmulhw  mm6,    [ebx+24]        ; Do the multiply, get the high word MULSH( m, n )
        psrlw   mm5,    15              ; Extract the sign bit of the numerator
        psrlw   mm7,    15              ; Extract the sign bit of the numerator
        psraw   mm4,    2               ; Right shift the quotient by two, 16+2 = 18 = N+L-1
        psraw   mm6,    2               ; Right shift the quotient by two, 16+2 = 18 = N+L-1
        paddw   mm4,    mm5             ; Add the sign bit
        paddw   mm6,    mm7             ; Add the sign bit
        movq  [edx+16], mm4             ; Done with 8 to 11
        movq  [edx+24], mm6             ; Done with 12 to 15
                                        ;

        add     ebx,    20h             ; 32 bytes, 16 integers
        add     edx,    20h             ;
        cmp     edx,    ecx             ; Did we reach end of block
        jl      DoDivide                ; Go back and to 16 to 63
                                        ;
                                        ; Clean Up
        pop     ebx                     ;
        pop     ecx                     ;
        pop     edx                     ;
    }       
}

// Deprecated by above functions

inline void CGS8x8Block::IntraQuantize()
{
    
    sint16 *pQntMatrix  = sQntMatrix;
    int    qScale       = 1 << CGSMacroBlock::QuantizationShifter;

    for( sint16 *pBlkMatrix  = sIntBlock; pBlkMatrix  < sIntBlock + 64;  )
    {
        *pBlkMatrix *= qScale;
        *pBlkMatrix /= *pQntMatrix;
        pBlkMatrix++;
        pQntMatrix++;
    }

}



inline void CGS8x8Block::IntraQuantizeAndPutBits( uint16 cc )
{
    sint16 *pScanMatrix = sScanMatrix; // This is set at the begining of the sequence.
    sint16 *pBlkMatrix  = sIntBlock;
    sint16 DCDCTDiff, iRun;
    sint16 iLevel;
    bool isNegative = false;

    iLevel = *pBlkMatrix;

#ifdef GSP4
    QuantizeP4();
#else
    Quantize();
#endif

    *pBlkMatrix = iLevel;

    if( iLevel < 0 )
    {
        isNegative = true;
        iLevel = -iLevel;
    }
    iLevel += (CGSMacroBlock::DCDivider >> 1);
    iLevel /= CGSMacroBlock::DCDivider;
    *pBlkMatrix = iLevel;
    if( isNegative )
    {
        *pBlkMatrix = -iLevel;
    }
    
    DCDCTDiff = sIntBlock[0] - CGSMacroBlock::dc_dct_pred[cc];
    CGSMacroBlock::dc_dct_pred[cc] = *pBlkMatrix;


    if( cc ) // Output DC per 7.2.1
    {
        CGSMacroBlock::pVLCoder->PutChrominDC( DCDCTDiff );
    }else
    {
        CGSMacroBlock::pVLCoder->PutLuminDC( DCDCTDiff );
    }

    pScanMatrix++; // Skip first, as we just scanned it!

    for ( iRun = 0; pScanMatrix < sScanMatrix+64; ) // Output AC per 7.2.2
    {
        pBlkMatrix   = sIntBlock  + *pScanMatrix++;

        iLevel = *pBlkMatrix++;

        if( iLevel )
        {
            CGSMacroBlock::pVLCoder->PutAC( iRun, iLevel );
            iRun = 0;
        }else
        {
            iRun++;
        }
    }

    CGSMacroBlock::pVLCoder->PutEndOfBlock();
}



void CGS8x8Block::IntraDequantize( sint16 *pSrc, sint16 *pDest )
{
    sint16 *pOutMatrix = pDest;
    sint16 *pQntMatrix = sQntMatrix;
    sint16  iDCValue   = (*pSrc) * CGSMacroBlock::DCDivider; // Scale the DC component

    __int64 shiftRight = CGSMacroBlock::QuantizationShifter;
/*
    for( ++pOutMatrix, ++pIntMatrix, ++pQntMatrix; pIntMatrix < pSrc + 64; )
    {
        *pOutMatrix++ = ( (*pIntMatrix++) * (*pQntMatrix++) ) / CGSMacroBlock::QuantizationScale;
    }
*/

    _asm
    {
        push    ebx
        push    ecx
        push    edx

        mov     ebx,    pSrc
        mov     ecx,    pQntMatrix
        mov     edx,    pDest

        ; // Get the First 32 DCT Coeffs in MMX Registers

        movq    mm0,      [ebx]
        movq    mm1,     8[ebx]
        movq    mm2,    16[ebx]
        movq    mm3,    24[ebx]
        movq    mm4,    32[ebx]
        movq    mm5,    40[ebx]
        movq    mm6,    48[ebx]
        movq    mm7,    56[ebx]

        ; // Multiply by the First 32 Quantization Matrix Values

        pmullw  mm0,      [ecx]
        pmullw  mm1,     8[ecx]
        pmullw  mm2,    16[ecx]
        pmullw  mm3,    24[ecx]
        pmullw  mm4,    32[ecx]
        pmullw  mm5,    40[ecx]
        pmullw  mm6,    48[ecx]
        pmullw  mm7,    56[ecx]

        ; // Divide by Quantizer Scale

        psraw   mm0, shiftRight
        psraw   mm1, shiftRight
        psraw   mm2, shiftRight
        psraw   mm3, shiftRight
        psraw   mm4, shiftRight
        psraw   mm5, shiftRight
        psraw   mm6, shiftRight
        psraw   mm7, shiftRight

        ; // Move them to the destination Matrix

        movq    [edx    ],mm0
        movq    [edx+  8],mm1
        movq    [edx+ 16],mm2
        movq    [edx+ 24],mm3
        movq    [edx+ 32],mm4
        movq    [edx+ 40],mm5
        movq    [edx+ 48],mm6
        movq    [edx+ 56],mm7


        ; // Get the Second 32 DCT Coeffs in MMX Registers

        add     ebx,    40h 
        add     ecx,    40h 
        add     edx,    40h 

        movq    mm0,      [ebx]
        movq    mm1,     8[ebx]
        movq    mm2,    16[ebx]
        movq    mm3,    24[ebx]
        movq    mm4,    32[ebx]
        movq    mm5,    40[ebx]
        movq    mm6,    48[ebx]
        movq    mm7,    56[ebx]

        ; // Multiply by the Quantization Matrix Values

        pmullw  mm0,      [ecx]
        pmullw  mm1,     8[ecx]
        pmullw  mm2,    16[ecx]
        pmullw  mm3,    24[ecx]
        pmullw  mm4,    32[ecx]
        pmullw  mm5,    40[ecx]
        pmullw  mm6,    48[ecx]
        pmullw  mm7,    56[ecx]

        ; // Divide by Quantizer Scale

        psraw   mm0, shiftRight
        psraw   mm1, shiftRight
        psraw   mm2, shiftRight
        psraw   mm3, shiftRight
        psraw   mm4, shiftRight
        psraw   mm5, shiftRight
        psraw   mm6, shiftRight
        psraw   mm7, shiftRight

        ; // Move them to the destination Matrix

        movq    [edx    ],mm0
        movq    [edx+  8],mm1
        movq    [edx+ 16],mm2
        movq    [edx+ 24],mm3
        movq    [edx+ 32],mm4
        movq    [edx+ 40],mm5
        movq    [edx+ 48],mm6
        movq    [edx+ 56],mm7

        pop     edx
        pop     ecx
        pop     ebx
    }

    *pDest = iDCValue; // Put back the DCT value!!!
    ippiDCT8x8Inv_16s_C1I( pDest );
}




void CGS8x8Block::IntraDequantizeP4( sint16 *pSrc, sint16 *pDest )
{
    sint16 *pOutMatrix = pDest;
    sint16 *pQntMatrix = sQntMatrix;
    sint16  iDCValue   = (*pSrc) * CGSMacroBlock::DCDivider; // Scale the DC component
    __int64 qScaleShifter = CGSMacroBlock::QuantizationShifter;

    _asm
    {
        push    ebx
        push    ecx
        push    edx

        mov     ebx,    pSrc
        mov     ecx,    pQntMatrix
        mov     edx,    pDest
        movq    mm0,    qScaleShifter
        movq2dq xmm7,   mm0

        ; // Get the First 32 DCT Coeffs in MMX Registers

        movdqa  xmm0,      [ebx]
        movdqa  xmm1,    16[ebx]
        movdqa  xmm2,    32[ebx]
        movdqa  xmm3,    48[ebx]
        movdqa  xmm4,    64[ebx]
        movdqa  xmm5,    80[ebx]
        movdqa  xmm6,    96[ebx]

        ; // Multiply by the First 32 Quantization Matrix Values

        pmullw  xmm0,      [ecx]
        pmullw  xmm1,    16[ecx]
        pmullw  xmm2,    32[ecx]
        pmullw  xmm3,    48[ecx]
        pmullw  xmm4,    64[ecx]
        pmullw  xmm5,    80[ecx]
        pmullw  xmm6,    96[ecx]


        ; // Divide by Quantizer Scale

        psraw   xmm0,   xmm7
        psraw   xmm1,   xmm7 
        psraw   xmm2,   xmm7
        psraw   xmm3,   xmm7 
        psraw   xmm4,   xmm7
        psraw   xmm5,   xmm7
        psraw   xmm6,   xmm7

        ; // Move them to the destination Matrix

        movdqa [edx    ],   xmm0
        movdqa [edx+ 16],   xmm1
        movdqa [edx+ 32],   xmm2
        movdqa [edx+ 48],   xmm3
        movdqa [edx+ 64],   xmm4
        movdqa [edx+ 80],   xmm5
        movdqa [edx+ 96],   xmm6

        movdqa  xmm0,       112[ebx]        ; Get the last row
        pmullw  xmm0,       112[ecx]        ; Multipy by Quantizer
        psraw   xmm0,       xmm7            ; Divide by qScale
        movdqa [edx+112],   xmm0            ; Save

        pop     edx
        pop     ecx
        pop     ebx
    }

    *pDest = iDCValue; // Put back the DCT value!!!
    ippiDCT8x8Inv_16s_C1I( pDest );
}















//------------------------------------------------------------------------------
// File: MacroBlk.cpp - Second Half
//
// Desc: Implementation File for Block8x8, we do it this way to make the above
//       functions inline.
//
// Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.
//------------------------------------------------------------------------------


  
  
void CGSMacroBlock::IntraQuantizeAndPutBits()
{
    int i;
    ubyte8 *pQntMatrix = (ubyte8 *)CGS8x8Block::sQntMatrix;

    for( i = 0; i < NumberOfBlocks; i++ )
    {
        CGS8x8Block Block( pBlocks[i] );
        Block.fDCT();
        Block.IntraQuantizeAndPutBits( cc[i] );
    }
}




void CGS8x8Block::Write(){};
void CGSMacroBlock::Write(){};
