//------------------------------------------------------------------------------
/* File: PPicture.cpp
  
   Desc: Implementation File for Three Plane Bitmap Image
  
   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------

#include <VLCoder.h>
#include <MotnCalc.h>
#include <PPicture.h>



void CGSPPicture::Predict( CGSPicture *pBeforeMe, CGSPicture *pAfterMe )
{
//    return;
    ubyte8  uMBRow, uMBCol;
    CGSMacroBlock *pMB = pMacroBlocks;
    int iTop, iLeft;

    for( uMBRow = 0; uMBRow < imageHeightInMacroBlocks; uMBRow++ )
    {
        iTop = 16 * uMBRow;
        for( uMBCol = 0; uMBCol < imageWidthInMacroBlocks; uMBCol++ )
        {
            iLeft = 16 * uMBCol;
            pMotionCalculator->CalcMotionByFullSearch( pMB, pBeforeMe->pYPlane, iLeft, iTop );
            pMB++;
        }
    }
}



void CGSPPicture::Write( ulong32 uPictureCount )
{
    uint16 uTemporalReference = (uint16)(uPictureCount%1024);
    ubyte8 uMBRow, uMBCol;
    ubyte8 ucQuantizerScaleCode = 8;// We are not doing any rate control.
    CGSMacroBlock *pMacroBlock = pMacroBlocks;

    PictureHeader.Write( *pBitWriter, P, uTemporalReference, 0xFFFF ); // VBV Delay is questionable!
    
    ExtPictureHeader.SetDCPrecision( 10 );
    ExtPictureHeader.SetFCodes( P, 3 );
    ExtPictureHeader.Write( *pBitWriter );

    for( uMBRow = 0; uMBRow < imageHeightInMacroBlocks; uMBRow++ )
    {
        int iAddressInc = 1;
        Slice.Write( *pBitWriter, uMBRow, ucQuantizerScaleCode );
        CGSMacroBlock::dc_dct_pred[0] = CGSMacroBlock::dc_dct_pred[1]
                                      = CGSMacroBlock::dc_dct_pred[2]
                                      = ExtPictureHeader.DCPredictorResetVal;

        for( uMBCol = 0; uMBCol < imageWidthInMacroBlocks; uMBCol++ )
        {
            pMacroBlock->Type = IntraBlock;
            if( pMacroBlock->Type == SkipBlock )
            {
                if( uMBCol == 0 || uMBCol == imageWidthInMacroBlocks-1 ) // Can't skip first or last
                {
                    pMacroBlock->Type = FwdBlock;
                }else
                {
                    iAddressInc++;
                    pMacroBlock++;
                    continue;
                }
            }

            pVLCoder->PutMacroBlockAddrInc( iAddressInc );

            switch( pMacroBlock->Type )
            {
                case IntraBlock:
                    pVLCoder->PutMacroBlockType( P, IntraBlock );
                    pMacroBlock->IntraQuantizeAndPutBits();
                    break;
                case FwdBlock:
                    pVLCoder->PutMacroBlockType( P, FwdBlock );
            }
            pMacroBlock++;
        }
    }
}


void CGSPPicture::ConstructImageFromMacroBlocks()
{
}


