//------------------------------------------------------------------------------
/* File: GSPicture.cpp
  
   Desc: Implementation File for Picture and its headers
  
   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------

#include <MacroBlk.h>
#include <BitWriter.h>
#include <MotnCalc.h>
#include <Picture.h>

CGSMotionCalculator *CGSPicture::pMotionCalculator = 0;
CGSBitWriter        *CGSPicture::pBitWriter        = 0;
CGSVLCoder          *CGSPicture::pVLCoder          = 0;


void CMPEGPictureHeader::Write( CGSBitWriter &BitWriter, PictureType ePictureType, uint16 iTRef, uint16 VBVDelay )
{
    BitWriter.AlignBits();

    BitWriter.PutBits( PictureStartCode,  32 );

    BitWriter.PutBits( iTRef,             10 ); //TemporalReference

    BitWriter.PutBits( ePictureType,       3 );

    BitWriter.PutBits( VBVDelay,          16 );

    if( ePictureType == P || ePictureType == B )
    {
        BitWriter.PutBits( 0, 1 );
        BitWriter.PutBits( 7, 3 );
        if( ePictureType == B )
        {
            BitWriter.PutBits( 0, 1 );
            BitWriter.PutBits( 7, 3 );
        }
    }

    BitWriter.PutBits( 0, 1 );
};


void CMPEGExtPictureHeader::Write( CGSBitWriter &BitWriter )
{
    BitWriter.AlignBits();
    BitWriter.PutBits( ExtStartCode,         32 );
    BitWriter.PutBits( ExtensionID,           4 );
    BitWriter.PutBits( FwdHorizonatlFCode,    4 );
    BitWriter.PutBits( FwdVerticalFCode,      4 );
    BitWriter.PutBits( BckdHorizonatlFCode,   4 );
    BitWriter.PutBits( BckVerticalFCode,      4 );
    BitWriter.PutBits( DCCoeffPrecision,      2 );
    BitWriter.PutBits( PictureStructure,      2 );
    BitWriter.PutBits( TopFirst,              1 );
    BitWriter.PutBits( FramePredictDCT,       1 );
    BitWriter.PutBits( ConcealmentVectors,    1 );
    BitWriter.PutBits( QScaleType,            1 );
    BitWriter.PutBits( IsIntraVLCFormat,      1 );
    BitWriter.PutBits( IsAlternateScan,       1 );
    BitWriter.PutBits( ShouldRepeatFirst,     1 );
    BitWriter.PutBits( IsChroma420,           1 );
    BitWriter.PutBits( IsProgressiveFrame,    1 );
    BitWriter.PutBits( CompsiteDisplayFlag,   1 );
}


void CMPEGExtPictureHeader::SetZigZagScan()
{ 
    IsAlternateScan = 0;
    CGS8x8Block::SetZigZagScan();
}

void CMPEGExtPictureHeader::SetAlternateScan()
{ 
    IsAlternateScan = 1;
    CGS8x8Block::SetAlternateScan();
}

void CMPEGExtPictureHeader::ClearIntraVLCFormat()
{ 
    IsIntraVLCFormat = 0;
}


void CMPEGExtPictureHeader::SetIntraVLCFormat()
{
    IsIntraVLCFormat = 1;
}



void CMPEGExtPictureHeader::SetDCPrecision( ubyte8 ucNumBits )
{
    if( ucNumBits > 10 )ucNumBits = 10;
    if( ucNumBits <  8 )ucNumBits =  8; // Less than 8 is sort of silly!
    ucNumBits -= 8;
    DCCoeffPrecision    = ucNumBits;
    DCPredictorResetVal = 128 << ucNumBits;
    CGSMacroBlock::SetDCPrecision( ucNumBits );
}



void CMPEGExtPictureHeader::SetFCodes( PictureType eType, ubyte8 ucCode )
{
    // We currently support only Frame Motion, no field motion

    switch( eType )
    {
        case I:
            FwdHorizonatlFCode  = 0xFF;       // MPEG2 - All bits set for I 
            FwdVerticalFCode    = 0xFF;       // MPEG2 - All bits set for I
            BckdHorizonatlFCode = 0xFF;       // MPEG2 - All bits set for I
            BckVerticalFCode    = 0xFF;       // MPEG2 - All bits set for I
            break;
        case P:
            FwdHorizonatlFCode  = ucCode;
            FwdVerticalFCode    = ucCode;
            BckdHorizonatlFCode = 0xFF;       // MPEG2 - All bits set for P
            BckVerticalFCode    = 0xFF;       // MPEG2 - All bits set for P
            break;
        case B:
            FwdHorizonatlFCode  = ucCode;
            FwdVerticalFCode    = ucCode;
            BckdHorizonatlFCode = ucCode;
            BckVerticalFCode    = ucCode;
            break;
    }

    CGSMotionCalculator::SetFCode( ucCode );
}




void CMPEGSlice::Write( CGSBitWriter &BitWriter, ubyte8 ucRow, ubyte8 ucScaleCode )
{
    BitWriter.AlignBits();
    BitWriter.PutBits( SliceStartCode+ucRow, 32 );
    BitWriter.PutBits( ucScaleCode,           5 );
    BitWriter.PutBits( 0,                     1 );
}



//------------------------------------------------------------------------------
// File: GSPicture.cpp
//
// Desc: Implementation of Picture Class
//
// Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.
//------------------------------------------------------------------------------


void CGSPicture::Finalize()
{
    if( isInitialized )
    {
        if( pMacroBlocks )
        {
            delete []pMacroBlocks;
            pMacroBlocks = 0;
        }
        if( pBlockMemoryRoot )
        {
            delete pBlockMemoryRoot;
            pBlockMemory = pBlockMemoryRoot = 0;
        }
        if( pPlanarImageRoot )
        {
            delete pPlanarImageRoot;
            pPlanarImage = pPlanarImageRoot = 0;
        }                
        isInitialized = false;
    }
}


            
void CGSPicture::CreateMacroBlockBlocks()
{
    sint16 *pCurrent = pBlockMemory;
    for( CGSMacroBlock *pMB = pMacroBlocks; pMB < pMacroBlocks+numberOfMacroBlocks; pMB++ )
    {
        for( int i = 0; i < CGSMacroBlock::NumberOfBlocks; i++ )
        {
            pMB->pBlocks[i] = pCurrent;
            pCurrent += 64;
        }
    }
}



void CGSPicture::InitializeMacroBlocks()
{
    CGSMacroBlock *pMB = pMacroBlocks;   

    for( int i = 0; i < imageHeightInMacroBlocks; i++ ) // For every MacroBlock Row...
    {
        ubyte8 *pTopLeftCornerY  = pYPlane  + (i * 16 * imageWidthInPixels); // Set Top Left Corner
        ubyte8 *pTopLeftCornerCb = pCbPlane + (i *  8 * imageWidthInPixels); // This only half as wide
        ubyte8 *pTopLeftCornerCr = pCrPlane + (i *  8 * imageWidthInPixels); // Also half as wide

        for( int j = 0; j < imageWidthInMacroBlocks; j++ )  // For Every Macro Block in that row
        {
            ubyte8 *pY  = pTopLeftCornerY;
            ubyte8 *pCb = pTopLeftCornerCb;
            ubyte8 *pCr = pTopLeftCornerCr;

            pMB->pBlockTL[0]   = pY;                        // Block Zero is right at the top
            pMB->pBlockTL[1]   = pY + 8;                    // Block 1 is 8 bytes to the right  
            pMB->pBlockTL[2]   = pY + 8*imageWidthInPixels; // Block 2 is 8 rows below in image plane
            pMB->pBlockTL[3]   = pMB->pBlockTL[2] + 8;      // Block 3 is 8 rows to the left of that

            pMB->pBlockTL[4]   = pCb;                       // Block 4 is the first Cb
            pMB->pBlockTL[5]   = pCr;                       // Block 5 is the first Cr 

            pMB->pBlockTL[6]   = pCb + 4*imageWidthInPixels;// Block 6 is 8 rows below Block 4, Cb half wide
            pMB->pBlockTL[7]   = pCr + 4*imageWidthInPixels;// Block 7 is 8 rows below Block 5, Cr half wide

            pTopLeftCornerY   += 16;                        // Start of next Macro block for Y
            pTopLeftCornerCb  +=  8;                        // Start of next Macro block for Cb, half wide
            pTopLeftCornerCr  +=  8;                        // Start of next Macro block for Cr, half wide
            pMB++;
        }
    }
}


void CGSPicture::Initialize( uint16 biWidth, uint16 biHeight )
{
    int sizeOfImage, sizeOfBlocks;

    imageWidthInPixels       = biWidth;
    imageWidthInBlocks       = (ubyte8)(imageWidthInPixels/8);
    imageWidthInMacroBlocks  = (ubyte8)(imageWidthInPixels/16);

    imageHeightInPixels      = biHeight;
    imageHeightInBlocks      = (ubyte8)(imageHeightInPixels/8);
    imageHeightInMacroBlocks = (ubyte8)(imageHeightInPixels/16);

    numberOfBlocks      = imageWidthInBlocks * imageHeightInBlocks;
    numberOfMacroBlocks = imageWidthInMacroBlocks * imageHeightInMacroBlocks;
    numberOfPixels      = imageWidthInPixels * imageHeightInPixels;

    sizeOfImage      = 2*numberOfPixels + 31;  // 1 + 1/2 + 1/2, 31 for alignment
    sizeOfBlocks     = numberOfMacroBlocks*CGSMacroBlock::NumberOfBlocks*8*8;
    sizeOfBlocks    += 32/sizeof(sint16); // I want 32 extra bytes for alignment

    pPlanarImageRoot = new ubyte8[sizeOfImage];
    pBlockMemoryRoot = new sint16[sizeOfBlocks];
    pMacroBlocks     = new CGSMacroBlock[numberOfMacroBlocks];

    pPlanarImage     = Align32( pPlanarImageRoot );
    pBlockMemory     = Align32( pBlockMemoryRoot );
    pYPlane          = pPlanarImage;
    pCbPlane         = pYPlane  + numberOfPixels;   // Offeset by one plane
    pCrPlane         = pCbPlane + numberOfPixels/2; // Offeset by half plane

    CGSMacroBlock::imageWidth = imageWidthInPixels; //Singleton, need to do it once per image.
    InitializeMacroBlocks();
    CreateMacroBlockBlocks();

    isInitialized = true;
}



void CGSPicture::Debug()
{
    for( CGSMacroBlock *pMB = pMacroBlocks; pMB < pMacroBlocks + numberOfMacroBlocks; pMB++ )
    {
        pMB->Write();
    }
}


void CGSPicture::SplitPlanes( ubyte8 *pRGBImage )
{
    register ubyte8 *pY   = pYPlane;
    register ubyte8 *pCb  = pCbPlane;
    register ubyte8 *pCr  = pCrPlane;
    register ubyte8 *pRGB = pRGBImage;

    register ubyte8 *pRGBEnd = pRGBImage + 2*numberOfPixels;  // YUV has only n+0.5n+0.5n pixels


    for( ;pRGB < pRGBEnd; )
    {
        *pY++  = *pRGB++;  // Y
        *pCb++ = *pRGB++;  // U
        *pY++  = *pRGB++;  // Y
        *pCr++ = *pRGB++;  //
    }    
}


void CGSPicture::OutputImage( ubyte8 *pRGBImage ) // This no longer works!
{
    ubyte8 *pY   = pYPlane;
    ubyte8 *pCb  = pCbPlane;
    ubyte8 *pCr  = pCrPlane;
    ubyte8 *pRGB = pRGBImage;

    ubyte8 *pRGBEnd = pRGBImage + 2*numberOfPixels;  // YUV has only n+0.5n+0.5n pixels


    for( ;pRGB < pRGBEnd; )
    {
        *pRGB   = 0;      // Test
        *pRGB++ = *pY++;  // Y
        *pRGB++ = 127;pCb++; // U
        *pRGB   =  0;     // Test
        *pRGB++ = *pY++;  // Y
        *pRGB++ = 127;pCr++; // V
    }    
}
