//------------------------------------------------------------------------------
/* File: Sequence.cpp
  
   Desc: Implementation File for Sequence Class
  
   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------

#include <Block8x8.h>
#include <MacroBlk.h>
#include <BitWriter.h>
#include <VLCoder.h>
#include <MotnCalc.h>
#include <Picture.h>
#include <Sequence.h>


void CMPEGSeqHeader::Write( CGSBitWriter &BitWriter )
{
    BitWriter.AlignBits();
    BitWriter.PutBits(   StartCode           , 32 );
    BitWriter.PutBits(   HorizontalSize      , 12 ); 
    BitWriter.PutBits(   VerticalSize        , 12 );
    BitWriter.PutBits(   AspectRatio         , 4  );
    BitWriter.PutBits(   FrameRateCode       , 4  );
    BitWriter.PutBits(   BitRate             , 18 ); 
    BitWriter.PutBits(   MarkerBit           , 1  );
    BitWriter.PutBits(   VBVBufferSize       , 10 );
    BitWriter.PutBits(   ConstraindParam     , 1  );
    BitWriter.PutBits(   LoadIntraQntMat     , 1  );
    BitWriter.PutBits(   LoadInterQntMat     , 1  );
}

void CMPEGExtSeqHeader::Write( CGSBitWriter &BitWriter )
{
    BitWriter.AlignBits();
    BitWriter.PutBits(   ExtStartCode         , 32 );
    BitWriter.PutBits(   SequenceID           , 4  );
    BitWriter.PutBits(   ProfileAndLevel      , 8  );
    BitWriter.PutBits(   IsProgressive        , 1  );
    BitWriter.PutBits(   ChromaFormat         , 2  );
    BitWriter.PutBits(   ExtHorizSize         , 2  );
    BitWriter.PutBits(   ExtVertSize          , 2  );
    BitWriter.PutBits(   ExtBitRate           , 12 );
    BitWriter.PutBits(   MarkerBit            , 1  );
    BitWriter.PutBits(   ExtVBVBuffSize       , 8  );
    BitWriter.PutBits(   IsLowDelay           , 1  );
    BitWriter.PutBits(   NFrameRate           , 2  );
    BitWriter.PutBits(   DFrameRate           , 5  );

};


void CMPEGExtDisplayHeader::Write( CGSBitWriter &BitWriter )
{
    BitWriter.AlignBits();
    BitWriter.PutBits( ExtStartCode            , 32 );
    BitWriter.PutBits( DisplaySequence         , 4  );
    BitWriter.PutBits( VideoFormat             , 3  );
    BitWriter.PutBits( ColorDescription        , 1  );
    BitWriter.PutBits( ColorPrimaries          , 8  );
    BitWriter.PutBits( TransCharacteristics    , 8  );
    BitWriter.PutBits( MatrixCoeffs            , 8  );
    BitWriter.PutBits( HorizontalSize          , 14 );
    BitWriter.PutBits( MarkerBit               , 1  );
    BitWriter.PutBits( VerticalSize            , 14 );
};


void CMPEGUserData::Write( CGSBitWriter &BitWriter )
{
    const sbyte8 *pBuffer = pUserData;

    BitWriter.AlignBits();
    BitWriter.PutBits( UserDataCode, 32 );
    while( *pBuffer )BitWriter.PutBits( *pBuffer++, 8 );
};

    
void CMPEGSeqEnd::Write( CGSBitWriter &BitWriter )
{
    BitWriter.AlignBits();
    BitWriter.PutBits( SequenceEndCode, 32 );
};


void CMPEGGOPHeader::Write( CGSBitWriter &BitWriter, ulong32 iTimeCode, bool isClosed )
{
    BitWriter.AlignBits();
    BitWriter.PutBits( GOPStartCode, 32 );
    BitWriter.PutBits( iTimeCode,    25 );
    BitWriter.PutBits( isClosed,      1 );
    BitWriter.PutBits( 0,             1 ); // Broken Link Code... set to zero in encoding, 6.3.8
};


void CGSSequence::Initialize( sint16 imageWidth, sint16 imageHeight )
{
    iPictureCount = 0;
    SequenceHeader.SetHorzSize( imageWidth  );
    SequenceHeader.SetVertSize( imageHeight );

    DisplayHeader.SetHorzSize( imageWidth  );
    DisplayHeader.SetVertSize( imageHeight );

    CGSMacroBlock::SetVLCoder( VLCoder );
    CGS8x8Block::FindMultiplier();
    CGSPicture::SetBitWriter( BitWriter );
    CGSPicture::SetVLCoder( VLCoder );
    CGSPicture::SetMotionCalculator( MotionCalculator );

    SequenceHeader.Write( BitWriter );
    ExtSequenceHeader.Write( BitWriter );
    DisplayHeader.Write( BitWriter );
    UserData.Write( BitWriter );
    GOPHeader.Write( BitWriter, 0, false );

    isInitialized = true;
}


void CGSSequence::Finalize()
{
    SequenceEnd.Write( BitWriter );
    isInitialized = false;
}
