//------------------------------------------------------------------------------
// File: GSEAlloc.h
/*
   Desc: GSEncoderAllocator Header File

   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------


class CGSEncoderInPin;
class CGSEncoder;

//----------------------------------------------------------------------------
// this is a special allocator that KNOWS the person who is creating it
// will only create one of them. It allocates CMediaSamples that only 
// reference the buffer location that is set in the pin's renderer's
// data variable
//----------------------------------------------------------------------------

class CGSEncoderAllocator : public CMemAllocator
{
    private:
        friend class CGSEncoderInPin;
        friend class CGSEncoder;

    protected:

        // our pin who created us
        //
        CGSEncoderInPin * m_pPin;

    public:

        CGSEncoderAllocator( CGSEncoderInPin * pParent, HRESULT *phr ) 
            : CMemAllocator( TEXT("GSEncoderAllocator"), NULL, phr ) 
            , m_pPin( pParent )
        {
        };

        ~CGSEncoderAllocator( )
        {
            // wipe out m_pBuffer before we try to delete it. It's not an allocated
            // buffer, and the default destructor will try to free it!
            m_pBuffer = NULL;
        }

        // we override this to tell whoever's upstream of us what kind of
        // properties we're going to demand to have
        //
        HRESULT GetAllocatorRequirements( ALLOCATOR_PROPERTIES *pProps );

        HRESULT Alloc( );

        void ReallyFree();
};
