//------------------------------------------------------------------------------
// File: GSEInPin.h
/*
   Desc: GSEncoderInPin Header File
  
   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------


class CGSEncoderInPin : public CTransInPlaceInputPin
{

    private:
        friend class CGSEncoderAllocator;
        friend class CGSEncoder;

        CGSEncoderAllocator * m_pPrivateAllocator;
        ALLOCATOR_PROPERTIES m_allocprops;
        BYTE * m_pBuffer;
        BOOL m_bMediaTypeChanged;

    protected:

        CGSEncoder * GSEncoder( ) { return (CGSEncoder*) m_pFilter; }
        HRESULT SetDeliveryBuffer( ALLOCATOR_PROPERTIES props, BYTE * m_pBuffer );

    public:

        CGSEncoderInPin( CTransInPlaceFilter * pFilter, HRESULT * pHr ) 
            : CTransInPlaceInputPin( TEXT("GSEncoderInputPin"), pFilter, pHr, L"Input" )
            , m_pPrivateAllocator( NULL )
            , m_pBuffer( NULL )
            , m_bMediaTypeChanged( FALSE )
        {
            memset( &m_allocprops, 0, sizeof( m_allocprops ) );
        }

        ~CGSEncoderInPin( )
        {
            if( m_pPrivateAllocator ) delete m_pPrivateAllocator;
        }

        // override to provide major media type for fast connects

        HRESULT GetMediaType( int iPosition, CMediaType *pMediaType );

        // override this or GetMediaType is never called

        STDMETHODIMP EnumMediaTypes( IEnumMediaTypes **ppEnum );

        // override this to refuse any allocators besides
        // the one the user wants, if this is set

        STDMETHODIMP NotifyAllocator( IMemAllocator *pAllocator, BOOL bReadOnly );

        // override this so we always return the special allocator, if necessary

        STDMETHODIMP GetAllocator( IMemAllocator **ppAllocator );

        HRESULT SetMediaType( const CMediaType *pmt );
};
