//------------------------------------------------------------------------------
// File: GSEncoder.h
/*
   Desc: GSEncoder Header File
  
   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Define GUID for the Encoder
//------------------------------------------------------------------------------


//GUID That Gireesh Generated for GSEncoder
// {9452A661-AE60-44a6-A51B-EBBE3500AD77}
DEFINE_GUID(CLSID_GSEncoder, 
0x9452a661, 0xae60, 0x44a6, 0xa5, 0x1b, 0xeb, 0xbe, 0x35, 0x0, 0xad, 0x77);

// we define a callback typedef for this example. 
// Normally, you would make the SampleEncoder 
// support a COM interface, and in one of it's methods
// you would pass in a pointer to a COM interface 
// for calling back. See the 
// DX8 documentation

typedef HRESULT (*SAMPLECALLBACK)
                                (
                                    IMediaSample * pSample, 
                                    REFERENCE_TIME * StartTime, 
                                    REFERENCE_TIME * StopTime,
                                    BOOL TypeChanged
                                 );
//GUID That Gireesh Generated for IGSEncoder
// {72D69D97-5147-4d66-9868-4EE982828F03}
DEFINE_GUID(IID_IGSEncoder, 
0x72d69d97, 0x5147, 0x4d66, 0x98, 0x68, 0x4e, 0xe9, 0x82, 0x82, 0x8f, 0x3);
// we define the interface the app can use to program us
MIDL_INTERFACE("72D69D97-5147-4d66-9868-4EE982828F03")
IGSEncoder : public IUnknown
{
    public:
        virtual HRESULT STDMETHODCALLTYPE SetAcceptedMediaType(const CMediaType *pType) = 0;
        
        virtual HRESULT STDMETHODCALLTYPE GetConnectedMediaType(AM_MEDIA_TYPE *pType) = 0;
        
        virtual HRESULT STDMETHODCALLTYPE SetCallback(SAMPLECALLBACK Callback) = 0;
        
        virtual HRESULT STDMETHODCALLTYPE SetDeliveryBuffer(ALLOCATOR_PROPERTIES props, BYTE *pBuffer) = 0;
};
        

#include <GSEAlloc.h>
#include <GSEInPin.h>
#include <GSEncCtl.h>

class CGSEncoder : public CTransInPlaceFilter
{

    private:
        friend class CGSEncoderInPin;
        friend class CGSEncoderAllocator;

    protected:

        CCritSec             m_Lock; // serialize access to our data
        CGSEncoderController m_Controller;  

        BOOL IsReadOnly( ) { return !m_bModifiesData; }

        // PURE, override this to ensure we get 
        // connected with the right media type

        // PURE, override this to callback 
        // the user when a sample is received
        HRESULT CheckInputType( const CMediaType * pmt );
        HRESULT Transform( IMediaSample * pms );
        HRESULT Receive( IMediaSample * pms );
        HRESULT StartStreaming();
        HRESULT StopStreaming();

    public:

        static CUnknown *WINAPI CreateInstance(LPUNKNOWN punk, HRESULT *phr);

        // Expose IGSEncoder
        STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void ** ppv);
        DECLARE_IUNKNOWN;

        CGSEncoder( IUnknown * pOuter, HRESULT * pHr, BOOL ModifiesData );

};
