//------------------------------------------------------------------------------
// File: GSEncCtl.cpp
/*
   Desc: Implementation File for the GSEncoderController
  
   Copyright (c) 2001-2002 Gireesh Sadasivan.  All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
//------------------------------------------------------------------------------

#include <BitWriter.h>
#include <VLCoder.h>
#include <MotnCalc.h>
#include <IPicture.h>
#include <PPicture.h>
#include <Sequence.h>
#include <GSEncCtl.h>


PictureType  CGSEncoderController::PictureTypeArray[15] =
{
    I, I, I, I, I,
    I, I, I, I, I,
    I, I, I, I, I
};



int CGSEncoderController::CreatePictures()
{
    for( int i = 0; i < numberOfPicturesInGOP; i++ )
    {
        switch( PictureTypeArray[i] )
        {
            case P:
                PictureArray[i] = new CGSPPicture();
                break;
            default:
                PictureArray[i] = new CGSIPicture();
                break;
        }

        if( PictureArray[i] )
        {
            PictureArray[i]->Initialize( imageWidth, imageHeight );
        }else
        {
            break;
        }
    }

//  Need to device a scheme to assure that the last picture is either a P or an I    
    return numberOfPicturesInGOP = i;
}



int CGSEncoderController::Initialize( sint16 biWidth, sint16 biHeight, const sbyte8 *pFileName )
{
    imageWidth            = biWidth;
    imageHeight           = biHeight;

    imagePlaneSize        = imageWidth*imageHeight;
    numberOfPicturesInGOP = 15;
    
    pDebugFile = fopen( "D:\\MyRecordings\\FirstFrame.BIT", "wb" );
    pMPEG2File = fopen( pFileName, "wb" );
    if( pMPEG2File )
    {
        isInitialized = true; // We need to do some clean up, at the very least
        pBitWriter = new CGSBitWriter( pMPEG2File );

        if( pBitWriter )
        {
            pVLCoder = new CGSVLCoder( *pBitWriter );
            if( pVLCoder )
            {
                pMotionCalculator = new CGSMotionCalculator( imageWidth, imageHeight );
                if( pMotionCalculator )
                {
                    pSequence  = new CGSSequence( *pBitWriter, *pVLCoder, *pMotionCalculator );
                    if( pSequence )
                    {
                        pSequence->Initialize( imageWidth, imageHeight );
                        if( CreatePictures() )
                        {
                            return 0;
                        }
                    }
                }
            }
        }
    }
    return -1;
}



void CGSEncoderController::FinalizePictures()
{
    for( int i = 0; i < numberOfPicturesInGOP; i++ )
    {
        pCurrentPicture = PictureArray[i];
        pCurrentPicture->Finalize();
        delete pCurrentPicture;
    }
}



void CGSEncoderController::Finalize()
{
    // First Close the MPEG2 Sequence..., close the MPEG2 File.

    pSequence->Finalize();
    FinalizePictures();

    delete pSequence;
    pSequence = 0;

    delete pVLCoder;
    pVLCoder = 0;

    delete pBitWriter;
    pBitWriter = 0;

    fclose( pMPEG2File );
    pMPEG2File = 0;

    if( pDebugFile )
    {
        fclose( pDebugFile );
        pDebugFile = 0;
    }

    isInitialized = false; // Prevent the dtor from screwing up!
}


void CGSEncoderController::CaptureSomeFrames( ubyte8 *pColorImage )
{
    if( pDebugFile )
    {
        switch( uPictureCount )
        {
            case 10:
                fwrite( &imageWidth,  sizeof( imageWidth  ), 1, pDebugFile );
                fwrite( &imageHeight, sizeof( imageHeight ), 1, pDebugFile );
            case 11:
            case 12:
                fwrite( pColorImage, sizeof( char ), imageWidth*imageHeight*2, pDebugFile );
        }
        if( uPictureCount == 12 )
        {
            fclose( pDebugFile );
            pDebugFile = 0;
        }
    }
}



void CGSEncoderController::ProcessImage( ubyte8 *pColorImage )
{
    int index = uPictureCount%numberOfPicturesInGOP;

    CGSPicture *pPictureBeforeMe, *pPictureAfterMe;

    pPictureBeforeMe = PictureArray[index-1];
    pCurrentPicture  = PictureArray[index];
    pPictureAfterMe  = PictureArray[index+1];

    if( index == 0 )pPictureBeforeMe = 0;
    if( index == numberOfPicturesInGOP )pPictureAfterMe  = 0;


    pCurrentPicture->SplitPlanes( pColorImage );
    pCurrentPicture->SetData(); // Fills MacroBlocks with data for Quantization of Blocks in I Pictures only
    pCurrentPicture->Predict( pPictureBeforeMe, pPictureAfterMe ); // Fill Macro Block Motion Information

#ifdef CONSOLE
    pCurrentPicture->Debug();
#endif

#ifdef GSOUTPUT
    CaptureSomeFrames( pColorImage );
#endif
   
    pCurrentPicture->Write( pSequence->GetPictureCount() );
    pCurrentPicture->ConstructImageFromMacroBlocks();
    pCurrentPicture->OutputImage( pColorImage );
    pSequence->IncrementPictureCount();

    uPictureCount++; // Increment Picture Count ... must fix this and GOP Header!
}